import os 

def tree(folder: str = os.getcwd(), depth: int = 0):
    # Edge case, "folder" is actually a file
    """ Recursively prints a visual representation of a given folder and all its subfolders and files.

    The printed representation is sorted in alphabetical order and does not take into account whether an entry is a folder or file.
    
    Parameters:
        folder : str, optional
            A folder which will have its contents printed. If not provided, the current working directory will be used.
        depth : int, optional
            Tracks the amount of recursive calls made in order to print lines for the parent directory. Should not be manually set.
    """
    
    # Box drawing characters used by Ubuntu. Gaps may appear on other terminals
    final_entry_delimiter = '\u2514\u2500\u2500'
    mid_sequence_delimiter = '\u251c\u2500\u2500'
    
    with os.scandir(folder) as folder_entries:
        alphabetized_entries = sorted(list(folder_entries), key=lambda e: e.name)
        
        try:
            first_entry, *middle_entries, last_entry = alphabetized_entries
        except ValueError:
            if len(alphabetized_entries) == 1:
                first_entry = alphabetized_entries[0]
                middle_entries = []
                last_entry = None
            #elif len(alphabetized_entries) == 2:
                #middle_entries = [] Need to set up a test case for when there are only 2 elements
         
        if len(alphabetized_entries) == 1:
            print_entry(first_entry, final_entry_delimiter, depth)
        else:
            print_entry(first_entry, mid_sequence_delimiter, depth)

        for entry in middle_entries:
            print_entry(entry, mid_sequence_delimiter, depth) 

        print_entry(last_entry, final_entry_delimiter, depth)

def print_entry(entry: os.DirEntry, delimiter: str, depth: int):
    """
    Helper method for tree. Prints a folder entry with a given delimiter if it's not empty and performs the recursive call for tree.

    """

    if entry is not None:
        root_delimiter = '\u2502'
        # 4 spaces initially because of the space taken by delimiters
        spaces = "   " * depth
        if depth > 0:
            print(f"{root_delimiter}{spaces}{delimiter} {entry.name}")
        else:
            print(f"{spaces}{delimiter} {entry.name}")
    if entry is not None and os.path.isdir(entry):
        tree(entry, depth + 1)   
tree() 
